@@ -53,6 +53,7 @@ sphere = function(x = 0, y = 0, z = 0, radius = 1, material = diffuse(),
   tibble::tibble(x = x, y = y, z = z, radius = radius, type = material$type, shape = "sphere",
                  properties = material$properties, velocity = list(velocity), 
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -128,6 +129,7 @@ cube = function(x = 0, y = 0, z = 0, width = 1, xwidth = 1, ywidth = 1, zwidth =
   tibble::tibble(x = x, y = y, z = z, radius = NA, type = material$type, shape = "box",
                  properties = list(boxinfo), velocity = list(velocity), 
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -187,6 +189,7 @@ xy_rect = function(x = 0, y = 0, z = 0, xwidth = 1, ywidth = 1,
   tibble::tibble(x = x, y = y, z = z, radius = NA, type = material$type, shape = "xy_rect",
                  properties = list(rectinfo), velocity = list(c(0, 0, 0)),
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -245,6 +248,7 @@ yz_rect = function(x = 0, y = 0, z = 0, ywidth = 1, zwidth = 1, material = diffu
   tibble::tibble(x = x, y = y, z = z, radius = NA, type = material$type, shape = "yz_rect",
                  properties = list(rectinfo), velocity = list(c(0, 0, 0)),
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -305,6 +309,7 @@ xz_rect = function(x = 0, xwidth = 1, z = 0, zwidth = 1, y = 0, material = diffu
                  type = material$type, shape = "xz_rect",
                  properties = list(rectinfo), velocity = list(c(0, 0, 0)),
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -392,6 +397,7 @@ triangle = function(v1 = c(1, 0, 0), v2 = c(0, 1, 0), v3 = c(-1, 0, 0),
                  type = material$type, shape = "triangle",
                  properties = list(info), velocity = list(c(0, 0, 0)),
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -461,6 +467,7 @@ disk = function(x = 0, y = 0, z = 0, radius = 1, inner_radius = 0, material = di
   tibble::tibble(x = x, y = y, z = z, radius = radius, type = material$type, shape = "disk",
                  properties = list(info), velocity = list(velocity), 
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -511,7 +518,7 @@ disk = function(x = 0, y = 0, z = 0, radius = 1, inner_radius = 0, material = di
 #'   add_object(obj_model(x = -1.8, y = -0.8, filename = r_obj() , 
 #'                        material = dielectric(color = "pink"))) %>%
 #'   add_object(sphere(z = 20, x = 20, y = 20, radius = 10,
-#'                     material = diffuse(lightintensity = 20, implicit_sample = TRUE))) %>%
+#'                     material = light(intensity = 20))) %>%
 #'   render_scene(parallel = TRUE, samples = 400, 
 #'                tonemap = "reinhold", aperture = 0.05, fov = 32, lookfrom = c(0, 2, 10))
 #' }
@@ -522,7 +529,7 @@ disk = function(x = 0, y = 0, z = 0, radius = 1, inner_radius = 0, material = di
 #'   add_object(obj_model(y = -0.8, filename = r_obj(), scale_obj = 2,
 #'                        material = diffuse(noise = TRUE, noiseintensity = 10,noisephase=45))) %>%
 #'   add_object(sphere(z = 20, x = 20, y = 20, radius = 10,
-#'                     material = diffuse(lightintensity = 10, implicit_sample = TRUE))) %>%
+#'                     material = light(intensity = 10))) %>%
 #'   render_scene(parallel = TRUE, samples = 400, ambient = TRUE, 
 #'                backgroundhigh="blue", backgroundlow="red",
 #'                aperture = 0.05, fov = 32, lookfrom = c(0, 2, 10),
@@ -545,6 +552,7 @@ obj_model = function(filename, x = 0, y = 0, z = 0, scale_obj = 1, texture = FAL
                  type = material$type, shape = shape,
                  properties = list(info), velocity = list(c(0, 0, 0)),
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -625,6 +633,7 @@ cylinder = function(x = 0, y = 0, z = 0, radius = 1, length = 1,
   tibble::tibble(x = x, y = y, z = z, radius = radius, type = material$type, shape = "cylinder",
                  properties = list(info), velocity = list(velocity), 
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -742,6 +751,7 @@ segment = function(start = c(0, -1, 0), end = c(0, 1, 0), radius = 1,
   tibble::tibble(x = x, y = y, z = z, radius = radius, type = material$type, shape = "cylinder",
                  properties = list(info), velocity = list(velocity), 
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
@@ -817,6 +827,7 @@ ellipsoid = function(x = 0, y = 0, z = 0, a = 1, b = 1, c = 1,
   tibble::tibble(x = x, y = y, z = z, radius = radius, type = material$type, shape = "ellipsoid",
                  properties = list(info), velocity = list(velocity), 
                  checkercolor = material$checkercolor, 
+                 gradient_color = material$gradient_color, gradient_transpose = material$gradient_transpose, 
                  noise = material$noise, noisephase = material$noisephase, 
                  noiseintensity = material$noiseintensity, noisecolor = material$noisecolor,
                  angle = list(angle), image = material$image, lightintensity = material$lightintensity,
