@@ -15,7 +15,7 @@
 #' @param lookat Default `c(0,0,0)`. Location where the camera is pointed.
 #' @param camera_up Default `c(0,1,0)`. Vector indicating the "up" position of the camera.
 #' @param aperture Default `0.1`. Aperture of the camera. Smaller numbers will increase depth of field, causing
-#' more blurring in areas not in focus.
+#' less blurring in areas not in focus.
 #' @param clamp_value Default `Inf`. If a bright light or a reflective material is in the scene, occasionally
 #' there will be bright spots that will not go away even with a large number of samples. These 
 #' can be removed (at the cost of slightly darkening the image) by setting this to a small number greater than 1. 
@@ -34,11 +34,16 @@
 #' Default `gamma` solely adjusts for gamma and clamps values greater than 1 to 1. 
 #' `reinhold` scales values by their individual color channels `color/(1+color)` and then performs the 
 #' gamma adjustment. `uncharted` uses the mapping developed for Uncharted 2 by John Hable. `hbd` uses an
-#' optimized formula by Jim Hejl and Richard Burgess-Dawson.
-#' @param backgroundimage Default `NULL`. 
+#' optimized formula by Jim Hejl and Richard Burgess-Dawson. Note: If set to anything other than `gamma`,
+#' objects with material `light()` may not be anti-aliased.
+#' @param environment_light Default `NULL`. An image to be used for the background for rays that escape
+#' the scene. Supports both HDR (`.hdr`) and low-dynamic range (`.png`, `.jpg`) images.
+#' @param rotate_env Default `0`. The number of degrees to rotate the environment map around the scene.
 #' @param parallel Default `FALSE`. If `TRUE`, it will use all available cores to render the image
 #'  (or the number specified in `options("cores")` if that option is not `NULL`).
 #' @param progress Default `TRUE` if interactive session, `FALSE` otherwise. 
+#' @param verbose Default `FALSE`. Prints information and timing information about scene
+#' construction and raytracing progress.
 #' @param debug Default `NULL`. If `bvh`, will return an image indicated the number of BVH lookups.
 #' @export
 #' @importFrom  grDevices col2rgb
@@ -102,7 +107,7 @@
 #'              samples=500)
 #' }
 #'                  
-#'#Increase the aperture to give more depth of field.
+#'#Increase the aperture to blur objects that are further from the focal plane.
 #' \donttest{
 #' render_scene(scene,lookfrom = c(7,1.5,10),lookat = c(0,0.5,0),fov=15,
 #'              aperture = 0.5,parallel=TRUE,samples=500)
@@ -126,12 +131,13 @@
 #'}
 #'}
 render_scene = function(scene, width = 400, height = 400, fov = 20, samples = 100, ambient_light = FALSE,
-                        lookfrom = c(0,1,10), lookat = c(0,0,0), camera_up = c(0,1,0), aperture = 0.1, clamp_value = Inf,
+                        lookfrom = c(0,1,10), lookat = c(0,0,0), camera_up = c(0,1,0), 
+                        aperture = 0.1, clamp_value = Inf,
                         filename = NULL, backgroundhigh = "#80b4ff",backgroundlow = "#ffffff",
                         shutteropen = 0.0, shutterclose = 1.0, focal_distance=NULL, ortho_dimensions = c(1,1),
                         tonemap ="gamma", parallel=TRUE,
-                        backgroundimage = NULL,
-                        progress = interactive(), debug = NULL) { 
+                        environment_light = NULL, rotate_env = 0,
+                        progress = interactive(), verbose = FALSE, debug = NULL) { 
   #Check if Cornell Box scene and set camera if user did not:
   if(!is.null(attr(scene,"cornell"))) {
     corn_message = "Setting default values for Cornell box: "
@@ -177,7 +183,7 @@ render_scene = function(scene, width = 400, height = 400, fov = 20, samples = 10
                           "sphere" = 1,"xy_rect" = 2, "xz_rect" = 3,"yz_rect" = 4,"box" = 5, "triangle" = 6, 
                           "obj" = 7, "objcolor" = 8, "disk" = 9, "cylinder" = 10, "ellipsoid" = 11))
   typevec = unlist(lapply(tolower(scene$type),switch,
-                          "diffuse" = 1,"metal" = 2,"dielectric" = 3, "oren-nayar" = 4))
+                          "diffuse" = 1,"metal" = 2,"dielectric" = 3, "oren-nayar" = 4, "light" = 5))
   sigmavec = unlist(scene$sigma)
   assertthat::assert_that(tonemap %in% c("gamma","reinhold","uncharted", "hbd"))
   toneval = switch(tonemap, "gamma" = 1,"reinhold" = 2,"uncharted" = 3,"hbd" = 4)
@@ -188,6 +194,12 @@ render_scene = function(scene, width = 400, height = 400, fov = 20, samples = 10
   checkeredlist = scene$checkercolor
   checkeredbool = purrr::map_lgl(checkeredlist,.f = ~all(!is.na(.x)))
   
+  #gradient handler
+  gradient_info = list()
+  gradient_info$gradient_colors = scene$gradient_color
+  gradient_info$isgradient = purrr::map_lgl(gradient_info$gradient_colors,.f = ~all(!is.na(.x)))
+  gradient_info$gradient_trans = scene$gradient_transpose
+  
   #noise handler
   noisebool = purrr::map_lgl(scene$noise, .f = ~.x > 0)
   noisevec = scene$noise
@@ -206,17 +218,16 @@ render_scene = function(scene, width = 400, height = 400, fov = 20, samples = 10
   flip_vec = scene$flipped
   
   #light handler
-  light_bool = !is.na(scene$lightintensity)
   light_prop_vec =  scene$lightintensity
   
-  if(!any(light_bool) && missing(ambient_light) && missing(backgroundimage)) {
+  if(!any(typevec == 5) && missing(ambient_light) && missing(environment_light)) {
     ambient_light = TRUE
   }
   
   #texture handler
   image_array_list = scene$image
   image_tex_bool = purrr::map_lgl(image_array_list,.f = ~is.array(.x))
-  temp_file_names = purrr::map_chr(image_tex_bool,.f = ~ifelse(.x, tempfile(),""))
+  temp_file_names = purrr::map_chr(image_tex_bool,.f = ~ifelse(.x, tempfile(fileext = ".png"),""))
   for(i in 1:length(image_array_list)) {
     if(image_tex_bool[i]) {
       if(dim(image_array_list[[i]])[3] == 4) {
@@ -263,9 +274,13 @@ render_scene = function(scene, width = 400, height = 400, fov = 20, samples = 10
   objbasedirvec = purrr::map_chr(objfilenamevec, dirname)
 
   #bg image handler
-  if(!is.null(backgroundimage)) {
+  if(!is.null(environment_light)) {
     hasbackground = TRUE
-    backgroundstring = path.expand(backgroundimage)
+    backgroundstring = path.expand(environment_light)
+    if(!file.exists(environment_light)) {
+      hasbackground = FALSE
+      warning("file '", environment_light, "' cannot be found, not using background image.")
+    }
   } else {
     hasbackground = FALSE
     backgroundstring = ""
@@ -316,10 +331,11 @@ render_scene = function(scene, width = 400, height = 400, fov = 20, samples = 10
                              bghigh = backgroundhigh, bglow = backgroundlow,
                              shutteropen = shutteropen, shutterclose = shutterclose,
                              ischeckered = checkeredbool, checkercolors = checkeredlist,
+                             gradient_info = gradient_info,
                              noise=noisevec,isnoise=noisebool,noisephase=noisephasevec, 
                              noiseintensity=noiseintvec, noisecolorlist = noisecolorlist,
                              angle = rot_angle_list, isimage = image_tex_bool, filelocation = temp_file_names,
-                             islight = light_bool, lightintensity = light_prop_vec,isflipped = flip_vec,
+                             lightintensity = light_prop_vec,isflipped = flip_vec,
                              focus_distance=focal_distance,
                              isvolume=fog_bool, voldensity = fog_vec , parallel=parallel,
                              implicit_sample = implicit_vec, order_rotation_list = order_rotation_list, clampval = clamp_value,
@@ -329,7 +345,8 @@ render_scene = function(scene, width = 400, height = 400, fov = 20, samples = 10
                              fileinfo = objfilenamevec, filebasedir = objbasedirvec, toneval = toneval,
                              progress_bar = progress, numbercores = numbercores, debugval = debugval,
                              hasbackground = hasbackground, background = backgroundstring, scale_list = scale_factor,
-                             ortho_dimensions = ortho_dimensions, sigmavec = sigmavec) 
+                             ortho_dimensions = ortho_dimensions, sigmavec = sigmavec, rotate_env = rotate_env,
+                             verbose = verbose) 
   full_array = array(0,c(ncol(rgb_mat$r),nrow(rgb_mat$r),3))
   full_array[,,1] = t(rgb_mat$r)
   full_array[,,2] = t(rgb_mat$g)
